<?php
if (!defined('ABSPATH'))
    exit;

class APIFW_Shipping_Label
{
    /**
     * The token.
     * @var     string
     * @access  public
     * @since   1.0.0
    */
    public $_token;

    /**
     * @var array
     * @access  public
     * @since   1.0.0
    */
    public $general_settings;

    /**
     * @var array
     * @access  public
     * @since   1.0.0
    */
    public $shipping_label_settings;

    /**
     * @var array
     * @access  public
     * @since   1.0.0
    */
    public $order_ids;

    /**
     * Constructor function.
     * @access  public
     * @return  void
     * @since   1.0.0
    */
    public function __construct( $order_ids = '' )
    {
        $this->_token = APIFW_TOKEN;
        if ( !$order_ids )
            return false;

        $this->order_ids = $order_ids;
        
        // getting general settings
        $this->general_settings = unserialize( get_option( $this->_token.'_general_settings' ) );

        // getting invoice settings
        $this->shipping_label_settings = unserialize( get_option( $this->_token.'_shipping_label_settings' ) );

        //calling shipping label pdf genereator
        $this->generate_shipping_label_pdf();
    }

    /**
     * Generating pdf file
     * @access  public
    */
    public function generate_shipping_label_pdf()
    {
        $rtl = $this->general_settings['rtl_support'];
        if( $rtl ){
            $ft_family = 'markazitext';
        } else {
            $ft_family = 'roboto';
        }

        require_once __DIR__ . '/vendor/autoload.php';
        $mpdf = new \Mpdf\Mpdf(
            [
                'format' => 'A4-P',
                'debugfonts'=> false,
                'mode' => 'utf-8',
                'autoScriptToLang'=>true,
                'autoLangToFont' => true,
                'default_font_size' => 13,
                //'default_font' => $ft_family,
                'margin_left' => 5,
                'margin_right' => 5,
                'margin_top' => 5,
                'margin_bottom' => 5,
            ]
        );
        $html = $this->get_shipping_label_html_template();
        $mpdf->WriteHTML( $html );
        $mpdf->Output( 'shipping-label.pdf', 'I' );
    }

    /**
     * Generating html template for pdf 
     * @access  public
    */
    public function get_shipping_label_html_template()
    {
        $order_ids = $this->order_ids;
        //general datas
        $company_name = $this->general_settings['company_name'];
        $company_logo = $this->general_settings['company_logo'];
        $sender_name = $this->general_settings['sender_name'];
        $sender_addr1 = $this->general_settings['sender_addr1'];
        $sender_addr2 = $this->general_settings['sender_addr2'];
        $sender_city = $this->general_settings['sender_city'];
        $sender_country = $this->general_settings['sender_country'];
        $sender_postal_code = $this->general_settings['sender_postal_code'];
        $sender_email = $this->general_settings['sender_email'];
        $sender_number = $this->general_settings['sender_number'];
        $tax_reg_no = $this->general_settings['gen_vat'];
        $footer_txt = $this->general_settings['gen_footer'];
        $rtl = $this->general_settings['rtl_support'];

        //html code for pdf
        $html = "<!DOCTYPE html>
        <html ".($rtl ? "dir='rtl'" : '' ).">
            <head>
                <meta charset='utf-8'>
                <title>".__( 'Shipping Label', 'pdf-invoices-and-packing-slips-for-woocommerce' )."</title>
                <style>
                    body {
                        position: relative;
                        color: #000000;
                    }
                    p {
                        margin: 0;
                        padding: 0;
                        font-size: 13px;
                        font-weight: normal;
                        line-height: 1.8;
                        color: #000000;
                    }
                    h4 {
                        line-height: 1.8;
                        margin: 0 0 4px;
                        padding: 0;
                        font-size: 16px;
                        font-weight: normal;
                        color: #000000;
                    }
                    .pdf_wrapper {
                        width: 93%;
                        padding: 20px 3.5% 10px;
                        background: #ffffff;
                        border: 1px solid #000;";   
                        if( count( $order_ids ) > 1 ):
                            $html .= "page-break-after: always;";
                        endif;
                        $html .= "
                    }
                    .top_sec {
                        width: 100%; 
                        clear:both;
                        margin-bottom: 10px;
                    }
                    .from_address_wrap {
                        float:left; 
                        width: 40.333%;
                    }
                    .from_address_inner {
                        width: 100%;
                        clear: both;
                    }
                    .shipping_address_wrap {
                        float:left; 
                        width: 40.333%;
                        padding-left: 2.5%;
                    }
                    .top_order_meta {
                        float:left;
                        width: 14.333%;
                        padding-left: 2.5%;
                        text-align: right;
                    }
                    .pdf_order_barcode {
                        text-align: center;
                    }
                    .pdf_order_barcode p {
                        margin-bottom: 5px;
                    }
                    .pdf_footer_wrap {
                        width: 100%;
                        margin-bottom: 12px;
                        margin-top: 10px;
                    }
                    .pdf_footer_wrap p {
                        font-size: 13px;
                        font-weight: normal;
                        color: #000000;
                        text-align: left;
                        line-height: 24px;
                        text-align: center;
                    }
                    /*.doc_rtl h2 {
                        line-height: 36px;
                        font-size: 24px;
                    }
                    .doc_rtl h4 {
                        font-size: 20px;
                    }
                    .doc_rtl p {
                        font-size: 17px;
                        line-height: 1.4;
                    }
                    .doc_rtl .pdf_footer_wrap p {
                        font-size: 18px;
                    }*/
                    .doc_rtl .from_address_wrap {
                        float: right;
                    }
                    .doc_rtl .shipping_address_wrap {
                        float: right; 
                        padding-left: 0%;
                        padding-right: 2.5%;
                    }
                    .doc_rtl .top_order_meta {
                        float: right;
                        padding-left: 0%;
                        padding-right: 2.5%;
                    }
                </style>
            </head>
            <body ".($rtl ? "dir='rtl'" : '' ).">";
                $count = 1;
                foreach( $order_ids as $order_id ):
                    $breake_avoid = '';
                    if( $count == count($order_ids) ){
                        $breake_avoid = 'page-break-after: avoid;';
                    }
                    $order_data = apifw_get_order_detail( $order_id );
                    $order_prd_infos = $order_data['items'];
                    $order_invoice_no = get_post_meta( $order_id, $this->_token.'_ord_invoice_no', true );
                    //order total weight calculation
                    $ord_total_weight = 0;
                    foreach( $order_prd_infos as $prd ) {
                        if( $prd['total_weight'] && is_numeric( $prd['total_weight'] ) ){
                            $ord_total_weight = $ord_total_weight + $prd['total_weight'];
                        }
                    }
                    $html .= "<div class='pdf_wrapper ".($rtl ? 'doc_rtl' : '' )."' style='".$breake_avoid."'>
                        <div class='top_sec'>";
                            $html .= "<div class='from_address_wrap'>";
                                $html .= "<h4>".__( 'From', 'pdf-invoices-and-packing-slips-for-woocommerce' )."</h4>";
                                $html .= "<div class='from_address_inner'>";
                                    if( $sender_name ):
                                        $html .= "<p>".$sender_name."</p>";
                                    endif;
                                    if( $sender_addr1 ):   
                                        $html .= "<p>".$sender_addr1."</p>";
                                    endif;
                                    if( $sender_addr2 ):
                                        $html .= "<p>".$sender_addr2."</p>";
                                    endif;
                                    if( $sender_city ):
                                        $html .= "<p>".$sender_city."</p>";
                                    endif;
                                    if( $sender_postal_code ):
                                        $html .= "<p>".$sender_postal_code."</p>";
                                    endif;
                                    if( $sender_country ):
                                        $html .= "<p>".$sender_country."</p>";
                                    endif;
                                    if( $sender_email ):
                                        $html .= "<p>".$sender_email."</p>";
                                    endif;
                                    if( $sender_number ):
                                        $html .= "<p>".$sender_number."</p>";
                                    endif;
                                $html .= "</div>
                            </div>
                            <div class='shipping_address_wrap'>";
                                $html .= "<h4>".__( 'To', 'pdf-invoices-and-packing-slips-for-woocommerce' )."</h4>";
                                $html .= "<div>
                                    <p>";
                                        if( $order_data['shipping_address']['first_name'] != '' ):
                                            $html .= $order_data['shipping_address']['first_name'];
                                        endif;
                                        if( $order_data['shipping_address']['last_name'] != '' ):
                                            $html .= " ".$order_data['shipping_address']['last_name']."<br/>";
                                        endif;
                                        if( $order_data['shipping_address']['company'] != '' ):
                                            $html .= $order_data['shipping_address']['company']."<br/>";
                                        endif;
                                        if( $order_data['shipping_address']['address_1'] != '' ):
                                            $html .= $order_data['shipping_address']['address_1']."<br/>";
                                        endif;
                                        if( $order_data['shipping_address']['address_2'] != '' ):
                                            $html .= $order_data['shipping_address']['address_2']."<br/>";
                                        endif;
                                        if( $order_data['shipping_address']['city'] != '' ):
                                            $html .= $order_data['shipping_address']['city']."<br/>";
                                        endif;
                                        if( $order_data['shipping_address']['formated_state'] != '' ):
                                            $html .= $order_data['shipping_address']['formated_state']."<br/>";
                                        endif;
                                        if( $order_data['shipping_address']['postcode'] != '' ):
                                            $html .= $order_data['shipping_address']['postcode']."<br/>";
                                        endif;
                                        if( $order_data['shipping_address']['formated_country'] != '' ):
                                            $html .= $order_data['shipping_address']['formated_country']."<br/>";
                                        endif;
                                        if( $order_data['billing_address']['email'] != '' ):
                                            $html .= __( 'Email', 'pdf-invoices-and-packing-slips-for-woocommerce' ).': '.$order_data['billing_address']['email']."<br/>";
                                        endif;
                                        if( $order_data['billing_address']['phone'] != '' ):
                                            $html .= __( 'Phone No', 'pdf-invoices-and-packing-slips-for-woocommerce' ).': '.$order_data['billing_address']['phone'];
                                        endif;
                                    $html .= "</p>
                                </div>
                            </div>
                            <div class='top_order_meta'>";
                                $html .= "<p>".__( 'Order No', 'pdf-invoices-and-packing-slips-for-woocommerce' ).': '.$order_data['order_number']."</p>";
                                $html .= "<p>".__( 'Weight', 'pdf-invoices-and-packing-slips-for-woocommerce' ).': '.($ord_total_weight > 0 ? $ord_total_weight.' '.$prd['weight_unit'] : __('N/A', 'pdf-invoices-and-packing-slips-for-woocommerce') )."</p>";
                            $html .= "</div>
                        </div>";
                        $html .= "<div class='pdf_order_barcode'>";
                            $html .= "<p>".__( 'Tracking Number', 'pdf-invoices-and-packing-slips-for-woocommerce' ).': '.$order_id."</p>";
                            $html .= "<barcode code='".$order_id."' type='UPCA' size='1' text='0' height='0.4' />";
                        $html .= "</div>";
                        if( $this->shipping_label_settings['add_footer'] == true && $footer_txt != '' ):
                            $html .= "<div class='pdf_footer_wrap'>
                                <p>".__($footer_txt, 'pdf-invoices-and-packing-slips-for-woocommerce')."</p>
                            </div>";
                        endif;
                    $html .= "</div>";
                    $count++;
                endforeach;
            $html .= "</body>
        </html>";
        //returning html
        return $html;
    }
}